::-webkit-scrollbar-track{-webkit-box-shadow:inset 0 0 6px rgba(0,0,0,.3);border-radius:10px;background-color:#f5f5f5}::-webkit-scrollbar{width:10px;background-color:#f5f5f5}::-webkit-scrollbar-thumb{border-radius:10px;-webkit-box-shadow:inset 0 0 6px rgba(0,0,0,.3);background-color:#555}span .v-chip__content{white-space:nowrap}html{overflow-y:auto!important}.v-menu--inline[data-v-6f45fbe6]{display:block}.chartjs-tooltip-key[data-v-6f45fbe6]{display:inline-block;width:10px;height:10px;margin-right:10px}html[data-v-258c6f62]{overflow-y:auto!important}.cm-s-made-of-code{font-size:1em;line-height:1.5em;font-family:inconsolata,monospace;letter-spacing:.3px;word-spacing:1px;background:rgba(9,9,22,.95);color:#f8f8f8}.cm-s-made-of-code .CodeMirror-lines{padding:8px 0}.cm-s-made-of-code .CodeMirror-gutters{box-shadow:1px 0 2px 0 rgba(0,0,0,.5);-webkit-box-shadow:1px 0 2px 0 rgba(0,0,0,.5);background-color:rgba(9,9,22,.95);padding-right:10px;z-index:3;border:none}.cm-s-made-of-code div.CodeMirror-cursor{border-left:3px solid #f8f8f8}.cm-s-made-of-code .CodeMirror-activeline-background{background:transparent}.cm-s-made-of-code .CodeMirror-selected{background:rgba(0,125,255,.5)}.cm-s-made-of-code .cm-comment{color:#54576b;background:transparent}.cm-s-made-of-code .cm-keyword{color:#ff3854}.cm-s-made-of-code .cm-string{color:#8fff58;background:rgba(16,38,34,.98)}.cm-s-made-of-code .cm-atom,.cm-s-made-of-code .cm-number,.cm-s-made-of-code .cm-property{color:#f1d950}.cm-s-made-of-code .cm-operator{color:#ff3854}.cm-s-made-of-code .CodeMirror-linenumber{color:#54576b}.cm-s-made-of-code{padding-top:10px;outline:thin solid rgba(51,51,51,.3)}.CodeMirror{height:100%;font-size:16px}div.CodeMirror-sizer>div{padding-left:10px}.CodeMirror-foldmarker{color:#333;text-shadow:#bfbfbf 1px 1px 2px,#bfbfbf -1px -1px 2px,#bfbfbf 1px -1px 2px,#bfbfbf -1px 1px 2px;font-family:arial;line-height:.3;cursor:pointer}.CodeMirror-foldgutter{width:.9em}.CodeMirror-foldgutter-folded,.CodeMirror-foldgutter-open{cursor:pointer}.CodeMirror-foldgutter-open:after{content:"\25BE"}.CodeMirror-foldgutter-folded:after{content:"\25B8"}.ansiStyle[data-v-327acab8]{background-color:#000;padding:10px}.desc[data-v-1ab7e8ff]{background-color:#8a8a8a;border:10px;border-right:20px}.ansiStyle[data-v-1ab7e8ff]{background-color:#000;padding:10px}.v-expansion-panel-content[data-v-2f922456]{background-color:#000}